package adaptors;

import net.minecraft.tileentity.TileEntity;

import net.minecraftforge.common.util.ForgeDirection;

import enetbridge.EnetBridge;
import factorization.api.IChargeConductor;

public class ChargeBlockDelegate extends TileEntityDelegate {
	public static TileEntityDelegate create(TileEntity te) {
		if (!isApplicable(te)) return null;

		try {
			return new ChargeBlockDelegate(te);
		} catch (Exception e) {
			EnetBridge.log.warn("Charge delegate init error: {}.", e.getMessage());
			return null;
		}
	}

	public static boolean isApplicable(TileEntity te) {
		return EnetBridge.hasCharge() && te instanceof IChargeConductor;
	}


	private ChargeBlockDelegate(TileEntity te) {
		super(te);

		this.base = (IChargeConductor) te;
	}

	@Override
	public void tick() {
	}

	@Override
	public boolean acceptsEnergyFrom(TileEntity emitter, ForgeDirection direction) {
		return !faulty;
	}

	@Override
	public boolean emitsEnergyTo(TileEntity receiver, ForgeDirection direction) {
		return !faulty;
	}

	@Override
	public double getDemandedEnergy() {
		if (faulty) return 0;

		return Math.max(0, EnetBridge.getTargetCharge() - base.getCharge().getValue()) * 1. / EnetBridge.getChargePerEu();
	}

	@Override
	public double getOfferedEnergy() {
		if (faulty) return 0;

		return Math.max(0, base.getCharge().getValue() - EnetBridge.getTargetCharge()) * EnetBridge.getEuPerCharge();
	}

	@Override
	public double injectEnergy(ForgeDirection directionFrom, double amount, double voltage) {
		amount *= EnetBridge.getChargePerEu();
		int iAmount = (int) amount;
		if (iAmount <= 0) return amount;

		amount -= iAmount;

		base.getCharge().addValue(iAmount);

		return amount * 1. / EnetBridge.getChargePerEu();
	}

	@Override
	public void drawEnergy(double amount) {
		amount -= base.getCharge().deplete((int) Math.ceil(amount / EnetBridge.getEuPerCharge())) * EnetBridge.getEuPerCharge();

		if (amount > 0) {
			EnetBridge.log.warn("Can't draw the full amount from {}, {} left over, disabling the delegate.", base, amount);
			faulty = true;
		}
	}


	private final IChargeConductor base;
	protected boolean faulty = false;
}
