package adaptors;

import java.util.EnumMap;
import java.util.Map;

import net.minecraft.tileentity.TileEntity;

import net.minecraftforge.common.util.ForgeDirection;

import buildcraft.api.mj.IBatteryIOObject;
import buildcraft.api.mj.IBatteryObject;
import buildcraft.api.mj.MjAPI;
import enetbridge.EnetBridge;
import ic2.api.energy.tile.IEnergySink;
import ic2.api.energy.tile.IEnergySource;

public class MjBlockDelegate extends TileEntityDelegate implements IEnergySink, IEnergySource {
	public static TileEntityDelegate create(TileEntity te) {
		if (!isApplicable(te)) return null;

		Map<ForgeDirection, IBatteryIOObject> batteries = getMjBatteries(te);
		if (batteries.isEmpty()) return null;

		try {
			return new MjBlockDelegate(te, batteries);
		} catch (Exception e) {
			EnetBridge.log.warn("MJ delegate init error: {}.", e.getMessage());
			return null;
		}
	}

	public static boolean isApplicable(TileEntity te) {
		return EnetBridge.hasMj() && !getMjBatteries(te).isEmpty();
	}

	private static Map<ForgeDirection, IBatteryIOObject> getMjBatteries(TileEntity te) {
		Map<ForgeDirection, IBatteryIOObject> ret = new EnumMap<ForgeDirection, IBatteryIOObject>(ForgeDirection.class);

		try {
			for (ForgeDirection dir : ForgeDirection.VALID_DIRECTIONS) {
				IBatteryObject battery = MjAPI.getMjBattery(te, MjAPI.DEFAULT_POWER_FRAMEWORK, dir);
				if (battery instanceof IBatteryIOObject) ret.put(dir, (IBatteryIOObject) battery);
			}
		} catch (Exception e) {
			EnetBridge.log.warn("MJ battery fetching error: {}.", e.getMessage());
			ret.clear();
		}

		return ret;
	}


	private MjBlockDelegate(TileEntity te, Map<ForgeDirection, IBatteryIOObject> batteries) {
		super(te);

		this.base = te;
		this.batteries = batteries;
	}

	@Override
	public void tick() {
		/*if (faulty) return;

		Map<ForgeDirection, IBatteryIOObject> newBatteries = new EnumMap<ForgeDirection, IBatteryIOObject>(ForgeDirection.class);

		for (ForgeDirection dir : ForgeDirection.VALID_DIRECTIONS) {
			IBatteryObject battery = MjAPI.getMjBattery(base, MjAPI.DEFAULT_POWER_FRAMEWORK, dir);
			if (battery instanceof IBatteryIOObject) newBatteries.put(dir, (IBatteryIOObject) battery);
		}

		if (!newBatteries.equals(batteries)) { this comparison always fails
			MinecraftForge.EVENT_BUS.post(new EnergyTileUnloadEvent(this));
			batteries.clear();
			batteries.putAll(newBatteries);
			MinecraftForge.EVENT_BUS.post(new EnergyTileLoadEvent(this));
		}*/
	}

	@Override
	public boolean acceptsEnergyFrom(TileEntity emitter, ForgeDirection direction) {
		if (faulty) return false;

		IBatteryIOObject battery = batteries.get(direction);
		if (battery == null) return false;

		return battery.canReceive();
	}

	@Override
	public boolean emitsEnergyTo(TileEntity receiver, ForgeDirection direction) {
		if (faulty) return false;

		IBatteryIOObject battery = batteries.get(direction);
		if (battery == null) return false;

		return battery.canSend();
	}

	@Override
	public double getDemandedEnergy() {
		if (faulty) return 0;

		double max = 0;

		for (IBatteryIOObject battery : batteries.values()) {
			double amount = battery.getEnergyRequested();
			if (amount > max) max = amount;
		}

		return max * 1. / EnetBridge.getMjPerEu();
	}

	@Override
	public double getOfferedEnergy() {
		if (faulty) return 0;

		double max = 0;

		// not supported in bc 6.0
		/*for (IBatteryIOObject battery : batteries.values()) {
			if (!battery.canSend()) continue;

			double amount = Math.min(battery.getEnergyStored(), battery.maxSendedPerCycle());
			if (amount > max) max = amount;
		}*/

		return max * EnetBridge.getEuPerMj();
	}

	@Override
	public double injectEnergy(ForgeDirection directionFrom, double amount, double voltage) {
		amount *= EnetBridge.getMjPerEu();

		if (directionFrom != ForgeDirection.UNKNOWN) {
			IBatteryIOObject battery = batteries.get(directionFrom);

			if (battery != null) amount -= battery.addEnergy(amount);
		}

		for (IBatteryIOObject battery : batteries.values()) {
			if (amount <= 0) break;
			amount -= battery.addEnergy(amount);
		}

		return amount * 1. / EnetBridge.getMjPerEu();
	}

	@Override
	public void drawEnergy(double amount) {
		// not supported in bc 6.0
	}

	private final TileEntity base;
	protected final Map<ForgeDirection, IBatteryIOObject> batteries;
	protected boolean faulty = false;
}
