package adaptors;

import net.minecraft.entity.EntityLivingBase;
import net.minecraft.item.ItemStack;

import cofh.api.energy.IEnergyContainerItem;
import enetbridge.EnetBridge;
import ic2.api.item.ElectricItem;
import ic2.api.item.IBackupElectricItemManager;

public class RfItemHandler implements IBackupElectricItemManager {
	public static boolean load() {
		try {
			ElectricItem.registerBackupManager(new RfItemHandler());
			return true;
		} catch (Exception e) {
			EnetBridge.log.warn("RF item handler init error: {}.", e.getMessage());
			return false;
		}
	}


	private RfItemHandler() {
	}

	@Override
	public boolean use(ItemStack stack, double amount, EntityLivingBase entity) {
		return ElectricItem.rawManager.use(stack, amount, entity);
	}

	@Override
	public String getToolTip(ItemStack stack) {
		return null;
	}

	@Override
	public double getCharge(ItemStack stack) {
		IEnergyContainerItem item = (IEnergyContainerItem) stack.getItem();
		return item.getEnergyStored(stack) / EnetBridge.getEuPerRf();
	}

	@Override
	public double discharge(ItemStack stack, double amount, int tier, boolean ignoreTransferLimit, boolean externally, boolean simulate) {
		int rf = (int) (amount / EnetBridge.getEuPerRf());
		if (rf <= 0) return 0;

		IEnergyContainerItem item = (IEnergyContainerItem) stack.getItem();
		int ret = item.extractEnergy(stack, rf, simulate);

		return Math.min(ret * EnetBridge.getEuPerRf(), amount);
	}

	@Override
	public void chargeFromArmor(ItemStack stack, EntityLivingBase entity) {
		ElectricItem.rawManager.chargeFromArmor(stack, entity);
	}

	@Override
	public double charge(ItemStack stack, double amount, int tier, boolean ignoreTransferLimit, boolean simulate) {
		int rf = (int) (amount * EnetBridge.getRfPerEu());
		if (rf <= 0) return 0;

		IEnergyContainerItem item = (IEnergyContainerItem) stack.getItem();
		double ret = item.receiveEnergy(stack, rf, simulate);

		return Math.min(ret / EnetBridge.getRfPerEu(), amount);
	}

	@Override
	public boolean canUse(ItemStack stack, double amount) {
		return ElectricItem.rawManager.canUse(stack, amount);
	}

	@Override
	public boolean handles(ItemStack stack) {
		return stack.getItem() instanceof IEnergyContainerItem;
	}
}
