package enetbridge;

import java.io.File;

import net.minecraft.client.Minecraft;

import cpw.mods.fml.common.FMLCommonHandler;
import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.EventHandler;
import cpw.mods.fml.common.event.FMLInitializationEvent;

import adaptors.RfItemHandler;
import ic2.core.util.Config;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


@Mod(
		modid = "EnetBridge",
		name = "IC2-x Energy Net Bridge",
		version = EnetBridge.VERSION,
		dependencies = "required-after:Forge@[10.13.0.1152,);"
				+ "required-after:IC2@[2.2.593,);"
				+ "after:CoFHAPI|energy;"
				+ "after:BuildCraftAPI|core;"
				+ "after:factorization"
		)
public class EnetBridge {
	public static final String VERSION = "1.4";

	public EnetBridge() {
		log = LogManager.getLogger("enetbridge");
	}

	@EventHandler
	public void onInit(FMLInitializationEvent event) {
		loadConfig();

		if (initRf()) {
			log.info("Loaded RF interface.");
		} else {
			log.info("Didn't load RF interface.");
		}

		if (initMj()) {
			log.info("Loaded MJ interface.");
		} else {
			log.info("Didn't load MJ interface.");
		}

		if (initCharge()) {
			log.info("Loaded Charge interface.");
		} else {
			log.info("Didn't load Charge interface.");
		}

		EventCallback.load();
	}

	public static boolean hasRf() {
		return hasRf;
	}

	public static boolean hasMj() {
		return hasMj;
	}

	public static boolean hasCharge() {
		return hasCharge;
	}

	public static double getRfPerEu() { // eu -> rf
		return rfPerEu;
	}

	public static double getEuPerRf() { // rf -> eu
		return euPerRf;
	}

	public static double getMjPerEu() { // eu -> mj
		return mjPerEu;
	}

	public static double getEuPerMj() { // mj -> eu
		return euPerMj;
	}

	public static double getChargePerEu() { // eu -> charge
		return chargePerEu;
	}

	public static double getEuPerCharge() { // charge -> eu
		return euPerCharge;
	}

	public static double getTargetCharge() {
		return targetCharge;
	}


	private void loadConfig() {
		Config config = new Config("ic2 general config");

		try {
			config.load(EnetBridge.class.getResourceAsStream("/assets/enetbridge/config/general.ini"));
		} catch (Exception e) {
			throw new RuntimeException("Error loading base config", e);
		}

		File configFile = new File(new File(getMcDir(), "config"), "enetbridge.ini");;

		try {
			if (configFile.exists()) config.load(configFile);
		} catch (Exception e) {
			throw new RuntimeException("Error loading user config", e);
		}

		try {
			config.save(configFile);
		} catch (Exception e) {
			throw new RuntimeException("Error saving user config", e);
		}

		rfPerEu = config.get("ratios/rfPerEu").getDouble();
		euPerRf = config.get("ratios/euPerRf").getDouble();
		mjPerEu = config.get("ratios/mjPerEu").getDouble();
		euPerMj = config.get("ratios/euPerMj").getDouble();
		chargePerEu = config.get("ratios/chargePerEu").getDouble();
		euPerCharge = config.get("ratios/euPerCharge").getDouble();
		targetCharge = config.get("misc/targetCharge").getInt();
	}

	private File getMcDir() {
		if (FMLCommonHandler.instance().getSide().isClient()) {
			return Minecraft.getMinecraft().mcDataDir;
		} else {
			return new File(".");
		}
	}

	private boolean initRf() {
		if (rfPerEu <= 0 && euPerRf <= 0) return false;

		try {
			Class.forName("cofh.api.energy.IEnergyHandler");
			Class.forName("cofh.api.energy.IEnergyContainerItem");
		} catch (ClassNotFoundException e) {
			log.debug("Can't find class: {}.", e.getMessage());
			return false;
		}

		if (RfItemHandler.load()) {
			log.info("Loaded RF item handler.");
		} else {
			log.info("Can't load RF item handler.");
		}

		hasRf = true;
		return true;
	}

	private boolean initMj() {
		if (rfPerEu <= 0 && euPerRf <= 0) return false;

		try {
			Class.forName("buildcraft.api.mj.MjAPI");
			Class.forName("buildcraft.api.mj.IBatteryIOObject");
			Class.forName("buildcraft.api.mj.IBatteryObject");
		} catch (ClassNotFoundException e) {
			log.debug("Can't find class: {}.", e.getMessage());
			return false;
		}

		hasMj = true;
		return true;
	}

	private boolean initCharge() {
		if (chargePerEu <= 0 && euPerCharge <= 0) return false;

		try {
			Class.forName("factorization.api.IChargeConductor");
			Class.forName("factorization.api.Charge");
		} catch (ClassNotFoundException e) {
			log.debug("Can't find class: {}.", e.getMessage());
			return false;
		}

		hasCharge = true;
		return true;
	}


	public static Logger log;

	private static double rfPerEu; // eu -> rf
	private static double euPerRf; // rf -> eu
	private static double mjPerEu; // eu -> mj
	private static double euPerMj; // mj -> eu
	private static double chargePerEu; // eu -> charge
	private static double euPerCharge; // charge -> eu
	private static int targetCharge;

	private static boolean hasRf = false;
	private static boolean hasMj = false;
	private static boolean hasCharge = false;
}
